import 'package:eufemia/eufemia.dart';
import 'package:flutter/material.dart';
import 'package:flutter/services.dart';
import 'package:flutter/widgets.dart';

/// {@category Components}
/// {@subCategory Bars}
/// A header or navigation bar from the Eufemia design system.
class Navbar<T> extends StatelessWidget implements PreferredSizeWidget {
  Navbar({
    Key key,
    this.title,
    this.actions,
    this.leading,
    this.bottom,
    this.centerTitle,
    this.automaticallyImplyLeading = true,
    this.usePlatformNavigation = false,
    this.result,
  })  : preferredSize = Size.fromHeight(
          bottom?.preferredSize?.height ?? 0.0 + kToolbarHeight,
        ),
        super(key: key);

  /// {@template flutter.widgets.AppBar.title}
  /// The primary widget displayed in the app bar.
  ///
  /// Typically a [Text] widget that contains a description of the current
  /// contents of the app.
  ///
  /// Becomes the middle component of the [NavigationToolBar] built by this widget.
  /// The [title]'s width is constrained to fit within the remaining space
  /// between the toolbar's [leading] and [actions] widgets. Its height is
  /// _not_ constrained. The [title] is vertically centered and clipped to fit
  /// within the toolbar, whose height is [toolbarHeight]. Typically this
  /// isn't noticeable because a simple [Text] [title] will fit within the
  /// toolbar by default. On the other hand, it is noticeable when a
  /// widget with an intrinsic height that is greater than [toolbarHeight]
  /// is used as the [title]. For example, when the height of an Image used
  /// as the [title] exceeds [toolbarHeight], it will be centered and
  /// clipped (top and bottom), which may be undesirable. In cases like this
  /// the height of the [title] widget can be constrained. For example:
  ///
  /// ```dart
  /// MaterialApp(
  ///   home: Scaffold(
  ///     appBar: AppBar(
  ///        title: SizedBox(
  ///        height: toolbarHeight,
  ///        child: child: Image.asset(logoAsset),
  ///      ),
  ///      toolbarHeight: toolbarHeight,
  ///   ),
  /// )
  /// ```
  /// {@endtemplate}
  final Widget title;

  /// {@template flutter.widgets.AppBar.actions}
  /// Widgets to display in a row after the [title] widget.
  ///
  /// Typically these widgets are [IconButton]s representing common operations.
  /// For less common operations, consider using a [PopupMenuButton] as the
  /// last action.
  ///
  /// The [actions] become the trailing component of the [NavigationToolBar] built
  /// by this widget. The height of each action is constrained to be no bigger
  /// than the [toolbarHeight].  final List<Widget> actions;
  /// {@endtemplate}
  final List<Widget> actions;

  /// {@template flutter.widgets.AppBar.leading}
  /// A widget to display before the [title].
  ///
  /// Typically the [leading] widget is an [Icon] or an [IconButton].
  ///
  /// Becomes the leading component of the [NavigationToolBar] built
  /// by this widget. The [leading] widget's width and height are constrained to
  /// be no bigger than [kToolbarHeight] and [toolbarHeight] respectively.
  ///
  /// If this is null and [automaticallyImplyLeading] is set to true, the
  /// [AppBar] will imply an appropriate widget. For example, if the [AppBar] is
  /// in a [Scaffold] that also has a [Drawer], the [Scaffold] will fill this
  /// widget with an [IconButton] that opens the drawer (using [Icons.menu]). If
  /// there's no [Drawer] and the parent [Navigator] can go back, the [AppBar]
  /// will use a [BackButton] that calls [Navigator.maybePop].
  ///
  /// The following code shows how the drawer button could be manually specified
  /// instead of relying on [automaticallyImplyLeading]:
  ///
  /// ```dart
  /// AppBar(
  ///   leading: Builder(
  ///     builder: (BuildContext context) {
  ///       return IconButton(
  ///         icon: const Icon(Icons.menu),
  ///         onPressed: () { ScaffoldMessenger.of(context).openDrawer(); },
  ///         tooltip: MaterialLocalizations.of(context).openAppDrawerTooltip,
  ///       );
  ///     },
  ///   ),
  /// )
  /// ```
  ///
  /// The [Builder] is used in this example to ensure that the `context` refers
  /// to that part of the subtree. That way this code snippet can be used even
  /// inside the very code that is creating the [Scaffold] (in which case,
  /// without the [Builder], the `context` wouldn't be able to see the
  /// [Scaffold], since it would refer to an ancestor of that widget).
  /// {@endtemplate}
  final Widget leading;

  /// {@template widgets}
  /// Controls whether we should try to imply the leading widget if null.
  ///
  /// If true and [leading] is null, automatically try to deduce what the leading
  /// widget should be. If false and [leading] is null, leading space is given to [title].
  /// If leading widget is not null, this parameter has no effect.
  /// {@endtemplate}
  final bool automaticallyImplyLeading;

  /// If true and [automaticallyImplyLeading] is true, the Navbar uses the
  /// underlying system's navigator instead of Flutter's navigator.
  final bool usePlatformNavigation;

  /// {@template flutter.widgets.AppBar.centerTitle}
  /// Whether the title should be centered.
  ///
  /// If this property is null, then [ThemeData.appBarTheme.centerTitle] is used,
  /// if that is also null, then value is adapted to the current [TargetPlatform].
  /// {@endtemplate}
  final bool centerTitle;

  /// {@template flutter.widgets.AppBar.bottom}
  /// This widget appears across the bottom of the app bar.
  ///
  /// Typically a [TabBar]. Only widgets that implement [PreferredSizeWidget] can
  /// be used at the bottom of an app bar.
  ///
  /// See also:
  ///
  ///  * [PreferredSize], which can be used to give an arbitrary widget a preferred size.
  final PreferredSizeWidget bottom;

  /// The value to pass through the navigator when the page containing this
  /// [Navbar] is popped of the navigation stack.
  final T result;

  @override
  Widget build(BuildContext context) {
    final palette = EufemiaPalette.of(context);
    final parentRoute = ModalRoute.of(context);

    final canPop = usePlatformNavigation || (parentRoute?.canPop ?? false);

    final style = getPlatformDefaultStyle(context).copyWith(
      color: palette.navbarTitle,
    );

    var icon = leading;

    if (icon == null) {
      if (automaticallyImplyLeading && canPop) {
        icon = GestureDetector(
          behavior: HitTestBehavior.opaque,
          child: Icon(EufemiaIcons.back_arrow, size: 16.0),
          onTap: () => usePlatformNavigation
              ? SystemNavigator.pop()
              : Navigator.of(context).pop(result),
        );
      }
    }

    return AppBar(
      title: DefaultTextStyle(
        child: AnimatedSwitcher(
          duration: Duration(milliseconds: 80),
          transitionBuilder: (child, animation) => FadeTransition(
            opacity: animation,
            child: child,
          ),
          child: title,
        ),
        style: style,
      ),
      actions: [
        if (actions?.isNotEmpty ?? false) ...{
          EufemiaRow(
            padding: EufemiaInsets.medium(EufemiaSides.right),
            children: actions,
          ),
        },
      ],
      bottom: bottom,
      centerTitle: centerTitle,
      leading: icon,
      titleSpacing:
          icon == null ? EufemiaSpace.medium.toPoints(context: context) : 0.0,
    );
  }

  @override
  final Size preferredSize;

  TextStyle getPlatformDefaultStyle(BuildContext context) {
    final typography = EufemiaTypography.of(context);

    switch (Theme.of(context).platform) {
      case TargetPlatform.iOS:
      case TargetPlatform.macOS:
        return typography.styles.bodyEmphasized.toTextStyle(context);
      default:
        return typography.styles.titleEmphasized.toTextStyle(context);
    }
  }
}
