import 'package:flutter/widgets.dart';
import 'package:provider/provider.dart';

import 'children_extensions.dart';
import 'insets.dart';
import 'padding.dart';
import 'space.dart';

/// {@category Spacing}
/// A supercharged [Flex] widget with Eufemia-specific properties like padding
/// and children spacing.
class EufemiaFlex extends StatelessWidget {
  /// {@macro flutter.rendering.Axis}
  final Axis direction;

  /// {@macro eufemia.spacing.insets}
  final EufemiaInsets padding;

  /// {@template eufemia.spacing.spaceBetween}
  /// The [EufemiaSpace] to inset between each child of this widget.
  /// {@endtemplate}
  final EufemiaSpace spaceBetween;

  /// {@macro flutter.widgets.children}
  final List<Widget> children;

  /// {@macro flutter.rendering.MainAxisAlignment}
  final MainAxisAlignment mainAxisAlignment;

  /// {@template flutter.rendering.MainAxisSize}
  /// How much space this widget should request along its main axis.
  /// {@endtemplate}
  final MainAxisSize mainAxisSize;

  /// {@template flutter.rendering.CrossAxisAlignment}
  /// How this widget's childrene should be distributed along the cross axis.
  /// {@endtemplate}
  final CrossAxisAlignment crossAxisAlignment;

  /// {@template flutter.rendering.VerticalDirection}
  /// Which direction along the vertical this widget's children should be laid
  /// out in.
  /// {@endtemplate}
  final VerticalDirection verticalDirection;

  /// {@template flutter.rendering.TextDirection}
  /// Which direction this widget's subtree of [Text] widgets should be laid out
  /// in.
  /// {@endtemplate}
  final TextDirection textDirection;

  /// {@template flutter.rendering.TextBaseLine}
  /// A horizontal line used for aligning text.
  /// {@endtemplate}
  final TextBaseline textBaseline;

  const EufemiaFlex({
    Key key,
    @required this.direction,
    this.mainAxisAlignment = MainAxisAlignment.start,
    this.mainAxisSize = MainAxisSize.max,
    this.crossAxisAlignment = CrossAxisAlignment.center,
    this.textDirection,
    this.verticalDirection = VerticalDirection.down,
    this.textBaseline,
    this.padding,
    this.spaceBetween,
    this.children = const <Widget>[],
  })  : assert(direction != null),
        assert(mainAxisAlignment != null),
        assert(mainAxisSize != null),
        assert(crossAxisAlignment != null),
        assert(verticalDirection != null),
        assert(crossAxisAlignment != CrossAxisAlignment.baseline ||
            textBaseline != null),
        super(key: key);

  @override
  Widget build(BuildContext context) {
    Widget result = Flex(
      direction: direction,
      mainAxisSize: mainAxisSize,
      crossAxisAlignment: crossAxisAlignment,
      textDirection: textDirection,
      verticalDirection: verticalDirection,
      mainAxisAlignment: mainAxisAlignment,
      textBaseline: textBaseline,
      children: spaceBetween != null && spaceBetween != EufemiaSpace.none
          ? children.spaced(spaceBetween)
          : children,
    );

    if (padding != null) {
      result = EufemiaPadding(
        padding: padding,
        child: result,
      );
    }

    return Provider.value(
      value: direction,
      child: result,
    );
  }
}

/// {@category Spacing}
/// A subclass of [EufemiaFlex] that lays its children out on the vertical axis.
class EufemiaColumn extends EufemiaFlex {
  const EufemiaColumn({
    Key key,
    MainAxisAlignment mainAxisAlignment = MainAxisAlignment.start,
    MainAxisSize mainAxisSize = MainAxisSize.max,
    CrossAxisAlignment crossAxisAlignment = CrossAxisAlignment.center,
    TextDirection textDirection,
    VerticalDirection verticalDirection = VerticalDirection.down,
    TextBaseline textBaseline,
    EufemiaInsets padding,
    EufemiaSpace spaceBetween,
    List<Widget> children = const <Widget>[],
  }) : super(
          children: children,
          key: key,
          padding: padding,
          spaceBetween: spaceBetween,
          direction: Axis.vertical,
          mainAxisAlignment: mainAxisAlignment,
          mainAxisSize: mainAxisSize,
          crossAxisAlignment: crossAxisAlignment,
          textDirection: textDirection,
          verticalDirection: verticalDirection,
          textBaseline: textBaseline,
        );
}

/// {@category Spacing}
/// A subclass of [EufemiaFlex] that lays its children out on the horizontal axis.
class EufemiaRow extends EufemiaFlex {
  const EufemiaRow({
    Key key,
    MainAxisAlignment mainAxisAlignment = MainAxisAlignment.start,
    MainAxisSize mainAxisSize = MainAxisSize.max,
    CrossAxisAlignment crossAxisAlignment = CrossAxisAlignment.center,
    TextDirection textDirection,
    VerticalDirection verticalDirection = VerticalDirection.down,
    TextBaseline textBaseline,
    EufemiaInsets padding,
    EufemiaSpace spaceBetween,
    List<Widget> children = const <Widget>[],
  }) : super(
          children: children,
          key: key,
          padding: padding,
          spaceBetween: spaceBetween,
          direction: Axis.horizontal,
          mainAxisAlignment: mainAxisAlignment,
          mainAxisSize: mainAxisSize,
          crossAxisAlignment: crossAxisAlignment,
          textDirection: textDirection,
          verticalDirection: verticalDirection,
          textBaseline: textBaseline,
        );
}
