import 'package:flutter/material.dart';

import 'space.dart';
import 'sides.dart';
import 'spacing.dart';
import 'spacing_data.dart';

/// {@category Spacing}
/// {@template eufemia.spacing.insets}
/// An immutable set of offsets in each of the four cardinal directions.
///
/// Typically used for an offset from each of the four sides of a box. For
/// example, the padding inside a box can be represented using this class.
///
/// The [EufemiaInsets] class specifies offsets in terms of visual edges, left,
/// top, right, and bottom. These values are not affected by the
/// [TextDirection].
///
/// Here are some examples of how to create [EdgeInsets] instances:
///
/// Typical eight-pixel margin on all sides:
///
/// ```dart
/// const EufemiaInsets.small()
/// ```
/// Eight pixel margin above and below, no horizontal margins:
///
/// ```dart
/// const EufemiaInsets.small(EufemiaSides.vertical)
/// ```
/// Left margin indent of 16 pixels:
///
/// ```dart
/// const EufemiaInsets.only(EufemiaSpace.medium, EufemiaSides.left)
/// ```
/// See also:
///  * [EufemiaPadding], a widget that accepts [EufemiaInsets] to describe its margins.
/// {@endtemplate}
class EufemiaInsets {
  /// The padding to add on the top.
  final EufemiaSpace top;

  /// The padding to add on the bottom.
  final EufemiaSpace bottom;

  /// The padding to add to the left.
  final EufemiaSpace left;

  /// The padding to add to the right.
  final EufemiaSpace right;

  /// An [EufemiaInsets] instance with no spacing.
  static const EufemiaInsets none = EufemiaInsets(all: EufemiaSpace.none);

  /// An [EufemiaInsets] instance with [EufemiaSpace.extraSmall] spacing on
  /// all sides.
  static const EufemiaInsets allExtraSmall =
      EufemiaInsets(all: EufemiaSpace.extraSmall);

  /// An [EufemiaInsets] instance with [EufemiaSpace.small] spacing on
  /// all sides.
  static const EufemiaInsets allSmall = EufemiaInsets(all: EufemiaSpace.small);

  /// An [EufemiaInsets] instance with [EufemiaSpace.medium] spacing on
  /// all sides.
  static const EufemiaInsets allMedium =
      EufemiaInsets(all: EufemiaSpace.medium);

  /// An [EufemiaInsets] instance with [EufemiaSpace.large] spacing on
  /// all sides.
  static const EufemiaInsets allLarge = EufemiaInsets(all: EufemiaSpace.large);

  /// An [EufemiaInsets] instance with [EufemiaSpace.extraLarge] spacing on
  /// all sides.
  static const EufemiaInsets allExtraLarge =
      EufemiaInsets(all: EufemiaSpace.extraLarge);

  /// An [EufemiaInsets] instance with [EufemiaSpace.extraExtraLarge] spacing on
  /// all sides.
  static const EufemiaInsets allExtraExtraLarge =
      EufemiaInsets(all: EufemiaSpace.extraExtraLarge);

  const EufemiaInsets({
    EufemiaSpace top,
    EufemiaSpace bottom,
    EufemiaSpace left,
    EufemiaSpace right,
    EufemiaSpace vertical,
    EufemiaSpace horizontal,
    EufemiaSpace all,
  })  : top = top ?? (vertical ?? (all ?? EufemiaSpace.none)),
        bottom = bottom ?? (vertical ?? (all ?? EufemiaSpace.none)),
        left = left ?? (horizontal ?? (all ?? EufemiaSpace.none)),
        right = right ?? (horizontal ?? (all ?? EufemiaSpace.none));

  /// Constructs an [EufemiaInsets] instance with [space] on only the provided
  /// [sides]. Defaults to [EufemiaSides.all].
  factory EufemiaInsets.only(EufemiaSpace space,
          [EufemiaSides sides = EufemiaSides.all]) =>
      EufemiaInsets(
        top: sides.hasTop ? space : EufemiaSpace.none,
        bottom: sides.hasBottom ? space : EufemiaSpace.none,
        right: sides.hasRight ? space : EufemiaSpace.none,
        left: sides.hasLeft ? space : EufemiaSpace.none,
      );

  /// Constructs an [EufemiaInsets] instance with [EufemiaSpace.extraSmall] on only the provided
  /// [sides]. Defaults to [EufemiaSides.all].
  factory EufemiaInsets.extraSmall([EufemiaSides sides = EufemiaSides.all]) =>
      EufemiaInsets.only(EufemiaSpace.extraSmall, sides);

  /// Constructs an [EufemiaInsets] instance with [EufemiaSpace.small] on only the provided
  /// [sides]. Defaults to [EufemiaSides.all].
  factory EufemiaInsets.small([EufemiaSides sides = EufemiaSides.all]) =>
      EufemiaInsets.only(EufemiaSpace.small, sides);

  /// Constructs an [EufemiaInsets] instance with [EufemiaSpace.medium] on only the provided
  /// [sides]. Defaults to [EufemiaSides.all].
  factory EufemiaInsets.medium([EufemiaSides sides = EufemiaSides.all]) =>
      EufemiaInsets.only(
        EufemiaSpace.medium,
        sides,
      );

  /// Constructs an [EufemiaInsets] instance with [EufemiaSpace.large] on only the provided
  /// [sides]. Defaults to [EufemiaSides.all].
  factory EufemiaInsets.large([EufemiaSides sides = EufemiaSides.all]) =>
      EufemiaInsets.only(
        EufemiaSpace.large,
        sides,
      );

  /// Constructs an [EufemiaInsets] instance with [EufemiaSpace.extraLarge] on only the provided
  /// [sides]. Defaults to [EufemiaSides.all].
  factory EufemiaInsets.extraLarge([EufemiaSides sides = EufemiaSides.all]) =>
      EufemiaInsets.only(
        EufemiaSpace.extraLarge,
        sides,
      );

  /// Constructs an [EufemiaInsets] instance with [EufemiaSpace.extraExtraLarge] on only the provided
  /// [sides]. Defaults to [EufemiaSides.all].
  factory EufemiaInsets.extraExtraLarge(
          [EufemiaSides sides = EufemiaSides.all]) =>
      EufemiaInsets.only(
        EufemiaSpace.extraExtraLarge,
        sides,
      );

  /// Returns the corresponding Flutter-friendly [EdgeInsets] from this
  /// [EufemiaInsets] instace.
  EdgeInsets toEdgeInsets({BuildContext context, EufemiaSpacingData data}) {
    assert(context != null || data != null);
    final spacing = data ?? EufemiaSpacing.of(context);
    return EdgeInsets.only(
      left: left.toPoints(data: spacing),
      right: right.toPoints(data: spacing),
      top: top.toPoints(data: spacing),
      bottom: bottom.toPoints(data: spacing),
    );
  }
}
