import 'package:flutter/material.dart';

import 'spacing_data.dart';

/// {@category Spacing}
/// Widget providing its children with [EufemiaSpacingData] as well as
/// methods for loading and updating the palette during runtime.
class EufemiaSpacing extends StatefulWidget {
  /// The [EufemiaSpacingData] data to be provided.
  final EufemiaSpacingData data;

  /// {@macro flutter.widgets.child}
  final Widget child;

  const EufemiaSpacing({
    Key key,
    @required this.data,
    this.child,
  }) : super(key: key);

  /// Retrieves the nearest instance of [EufemiaSpacingData] in the tree.
  static EufemiaSpacingData of(BuildContext context) {
    final provider =
        context.dependOnInheritedWidgetOfExactType<EufemiaSpacingProvider>();
    if (provider == null) return EufemiaSpacingData.fallback();
    return provider.value;
  }

  static _EufemiaSpacingState _findState(BuildContext context) {
    final state = context.findAncestorStateOfType<_EufemiaSpacingState>();
    if (state == null) {
      throw Exception('No EufemiaSpacing found in the widget tree');
    }
    return state;
  }

  /// Updates the [data] with the given [data].
  static void update(BuildContext context, EufemiaSpacingData data) {
    _findState(context).update(data);
  }

  @override
  _EufemiaSpacingState createState() => _EufemiaSpacingState();
}

class _EufemiaSpacingState extends State<EufemiaSpacing> {
  EufemiaSpacingData _data;

  @override
  void initState() {
    _data = widget.data;
    super.initState();
  }

  void update(EufemiaSpacingData spacing) {
    setState(() => _data = spacing);
  }

  @override
  Widget build(BuildContext context) {
    return EufemiaSpacingProvider(
      value: _data ?? EufemiaSpacingData.fallback(),
      child: widget.child,
    );
  }
}

/// {@category Spacing}
/// A provider wrapper around [EufemiaSpacingData] allowing its children
/// to be notified and rebuilt when the spacing changes.
class EufemiaSpacingProvider extends InheritedWidget {
  final EufemiaSpacingData value;

  EufemiaSpacingProvider({
    Key key,
    Widget child,
    this.value,
  }) : super(
          key: key,
          child: child,
        );

  @override
  bool updateShouldNotify(EufemiaSpacingProvider oldWidget) {
    return value != oldWidget.value;
  }
}
